#include <ESP8266WiFi.h>

// Protocol format: {[comand],[var1],[var2],...,[varn]}
// only for this device:
// Get brightness: {BRI?}, expected answer {BRI!,[%value]}
// Set brightnes: {BRI,[%value]}, expected answer {BRI!,[%value]}
// Answer for invalid requests:{ERR}
const char beginChar='{';
const char endChar='}';
const char separator=',';
const char getChar='\?';

const int outputPin = 2;
const int buttonPin = 0;

const char* ssid     = "yourSSID";
const char* WiFiPassword = "yourpassword";
const int   serverPort = 23;

WiFiServer wifiServer(serverPort);

const int maxBrightness=1023;
const int minBrightness=0;
int lowBright=200;
int mediumBright=600;
int brightnessStep=200;
int brightness = minBrightness;

const unsigned int debounceInterval = 300;
int lastPushTime = 0;
int pushButtonState;
int lastPushButtonState=HIGH; 
bool debouncing=false;

String dataReceived;
const unsigned int maxDataLenght=24;

unsigned int charCount=0;
bool validString = false;
bool eolChar = false;
bool checking=true;
 
void setup() {
  Serial.begin(115200);
  pinMode(outputPin,OUTPUT);
  pinMode(buttonPin, INPUT);
  delay(10);
  analogWrite(outputPin, minBrightness);
  WiFi.begin(ssid, WiFiPassword);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.println("Connecting..");
  }

  Serial.print("Connected to WiFi. IP:");
  Serial.println(WiFi.localIP());
  wifiServer.begin();
  Serial.setDebugOutput(true); 
}


void launchErrorMessage(WiFiClient *currentClient, String errMessage){
  currentClient->println("{ERR}");
  Serial.print("Invalid request: ");
  Serial.println(errMessage);
}

void readStringMessage(String message, int *ptrCurrentBrightness, WiFiClient *currentClient){
  int valueExtracted;
  String textData="";
  bool error=false;
  if(message.substring(1).startsWith("BRI")){
    //TODO: need a check for separator
    if(message.charAt(4)!=getChar){
      int i=5;
      bool numberFound=false;
      do {
        if(message.charAt(i)!=separator){
           if(isDigit(message.charAt(i))){
            textData+=message.charAt(i);
            numberFound=true;
           }else if(message.charAt(i)!=endChar){
            error=true;
           }
        } 
        i++;
      }while((message.charAt(i)!=endChar && message.charAt(i)!=separator) && numberFound && !error);
      
      if (numberFound){
        valueExtracted=textData.toInt();
        if(valueExtracted<0){
          valueExtracted=0;
        } else if(valueExtracted>1023){
          valueExtracted=1023;
        }
        *ptrCurrentBrightness=valueExtracted;
        analogWrite(outputPin, *ptrCurrentBrightness);
      } 
    }
    if ( (message.charAt(4)==getChar || (message.charAt(4)==separator && isDigit(message.charAt(5)))) && !error){
      currentClient->print("{BRI!,");
      currentClient->print(*ptrCurrentBrightness);
      currentClient->println("}");
      Serial.print("{BRI!,");
      Serial.print(*ptrCurrentBrightness);
      Serial.println("}");
    }
  } else {
    error=true;
  }
  if (error){
    launchErrorMessage(currentClient, message);
  }
}

void pulseDetection(){
  if((millis()-lastPushTime)>debounceInterval){
    debouncing=false;
  }
  if(digitalRead(buttonPin)==LOW && !debouncing){
    debouncing=true;
    lastPushTime=millis();
    if(brightness>0){
      if(brightness < lowBright){
        brightness=lowBright;
      } else if (brightness < mediumBright) {
        brightness=mediumBright;
      } else if (brightness < maxBrightness){
        brightness=maxBrightness;
      }else{
        brightness=minBrightness;
      }
     }else{
      brightness=lowBright; 
     }
    }
    analogWrite(outputPin, brightness);
  }

void loop() { 
    
  pulseDetection();
  WiFiClient client = wifiServer.available();
  
  if (client) {
    while (client.connected()) {
       pulseDetection();
      do {
        char c=client.read();
        dataReceived+=c;
        if((isAlpha(c)||isDigit(c)||c==beginChar||c==endChar||c==separator||c==getChar)&&checking){
          charCount++;
          switch(charCount){
            case 1:
                    if(c!=beginChar){
                      validString=false;
                      checking=false;
                    } else {
                      validString=true;
                      checking=true;
                    }
                    break;
            case maxDataLenght:
                    if(c!=endChar){
                      validString=false;
                      checking=true;
                    } else {
                      validString=true;
                      checking=false;
                    }
                    break;
            default:
                    if(c==endChar){
                      eolChar=true;
                      validString=true;
                      checking=false;
                    } else if (isDigit(c)||c==separator){
                      validString=true;
                      checking=true;
                    } else { //Redundant????
                      validString=false;
                      checking=true;
                    }
                    break;
          }
        } else {
          dataReceived="";
          charCount=0;
          validString=false;
          checking=true;
          eolChar=false;
          client.flush();
        }
        
      } while (client.available()>0 && charCount<maxDataLenght && !eolChar && validString && checking);
      
      if(validString && !checking && charCount>0){
        dataReceived.toUpperCase();
        readStringMessage(dataReceived,&brightness,&client);
        dataReceived="";
        charCount=0;
      } else if(!validString && !checking){
        dataReceived="";
        charCount=0;
        client.flush();
      }
      delay(10);
    }
    client.stop();
    Serial.println("Client disconnected");
  }
  
} //loop end
